#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

const int Analog_Pin=1;
const int Digital_Pin=2;
int WARMUP_TIME = 30;
unsigned long time;
float alcohol_value;

// set the pins for the components
int buzzer = 4;
int green_LED = 5;
int red_LED = 3;

LiquidCrystal_I2C lcd(0x27, 16, 2);

void setup() {
  lcd.begin();
  Serial.begin(9600);
  
  pinMode(buzzer, OUTPUT);
  pinMode(green_LED, OUTPUT);
  pinMode(red_LED, OUTPUT);


  // Turn on the blacklight and print an instructive message
  lcd.backlight();
  lcd.setCursor (0,0);
  lcd.print("Detect alcohol.");
  lcd.setCursor (0,1);
  lcd.print("Breath here!");
  delay(2000);
  lcd.clear();
}

// the sound when the alcohol limit is exceeded
void buzzer_sound() { 
  for (int i = 0; i < 5; i++){
   digitalWrite(red_LED, HIGH);
   digitalWrite(green_LED, LOW);
   delay(300);
   noTone(buzzer);
   tone(buzzer,400,500);
   digitalWrite(green_LED, LOW);
   digitalWrite(red_LED, HIGH);
   delay(300);
   noTone(buzzer);
   tone(buzzer,300,300);
  }

}

// extract alcohol value with the alcohol sensor
void calculate_alcohol_value()
{
  alcohol_value = 0;
  for(int i = 0; i < 10; i++)
  {
    alcohol_value = alcohol_value + analogRead(Analog_Pin);
    // calibrate the value
    alcohol_value -= 410;
    delay(10);
  }
  Serial.println(alcohol_value);
}

// print the status of the alcohol concentration in the air
void print_status(float alcohol_concentration)
{
  if(alcohol_concentration > 0.8)
  {
     Serial.println("You are drunk!");
     lcd.setCursor(0, 0);
      lcd.print(alcohol_concentration, 4);
      lcd.print(" mg/L        ");
      lcd.setCursor(0, 1);
      lcd.print("You are drunk!");
      buzzer_sound();
      digitalWrite(green_LED, LOW);
      digitalWrite(red_LED, LOW);
      lcd.clear();
  }
  else 
  {
    Serial.println("You are sober!");
      lcd.setCursor(0, 0);
      lcd.print(alcohol_concentration, 4);
      lcd.print(" mg/L        ");
      lcd.setCursor(0, 1);
      lcd.print("You are sober!");
      digitalWrite(green_LED, HIGH);
      delay(2000);
      lcd.clear();
  }
}

void loop()
{
  calculate_alcohol_value();
  float value = ((alcohol_value)/10) * (5.0/1024.0);
  float alcohol_concentration= 0.67 * value;

  // wait a time to warm up the sensor
  time = millis()/1000;
  if(time <= WARMUP_TIME)
  {
    time = map(time, 0, WARMUP_TIME, 0, 100);

    lcd.setCursor(0, 0);
    lcd.print("Warming up...");
    lcd.setCursor(0, 1);
    lcd.print(time);
    lcd.print("% complete");
    delay(2000);
    lcd.clear();
  }else
  {
     print_status(alcohol_concentration);
  }
  delay (100);
}
